@@ -33,7 +33,6 @@
 #include "cc-background-item.h"
 #include "cc-background-preview.h"
 #include "cc-background-resources.h"
-#include "cc-background-xml.h"
 
 #define WP_PATH_ID "org.gnome.desktop.background"
 #define WP_LOCK_PATH_ID "org.gnome.desktop.screensaver"
@@ -46,6 +45,7 @@
 
 #define INTERFACE_PATH_ID "org.gnome.desktop.interface"
 #define INTERFACE_COLOR_SCHEME_KEY "color-scheme"
+#define INTERFACE_ACCENT_COLOR_KEY "accent-color"
 
 struct _CcBackgroundPanel
 {
@@ -61,6 +61,7 @@ struct _CcBackgroundPanel
 
   CcBackgroundItem *current_background;
 
+  GtkWidget *accent_box;
   CcBackgroundChooser *background_chooser;
   CcBackgroundPreview *default_preview;
   CcBackgroundPreview *dark_preview;
@@ -70,6 +71,8 @@ struct _CcBackgroundPanel
 
 CC_PANEL_REGISTER (CcBackgroundPanel, cc_background_panel)
 
+static void on_settings_changed (CcBackgroundPanel *self);
+
 static void
 load_custom_css (CcBackgroundPanel *self)
 {
@@ -82,28 +85,6 @@ load_custom_css (CcBackgroundPanel *self)
                                               GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
 }
 
-static void
-reload_color_scheme_toggles (CcBackgroundPanel *self)
-{
-  GDesktopColorScheme scheme;
-
-  scheme = g_settings_get_enum (self->interface_settings, INTERFACE_COLOR_SCHEME_KEY);
-
-  if (scheme == G_DESKTOP_COLOR_SCHEME_DEFAULT)
-    {
-      gtk_toggle_button_set_active (self->default_toggle, TRUE);
-    }
-  else if (scheme == G_DESKTOP_COLOR_SCHEME_PREFER_DARK)
-    {
-      gtk_toggle_button_set_active (self->dark_toggle, TRUE);
-    }
-  else
-    {
-      gtk_toggle_button_set_active (self->default_toggle, FALSE);
-      gtk_toggle_button_set_active (self->dark_toggle, FALSE);
-    }
-}
-
 static void
 transition_screen (CcBackgroundPanel *self)
 {
@@ -124,6 +105,155 @@ transition_screen (CcBackgroundPanel *self)
     g_warning ("Couldn't transition screen: %s", error->message);
 }
 
+static void
+on_accent_color_toggled_cb (CcBackgroundPanel *self,
+                            GtkToggleButton   *toggle)
+{
+  GDesktopAccentColor accent_color_from_key;
+  GDesktopAccentColor accent_color = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (toggle), "accent-color"));
+
+  if (!gtk_toggle_button_get_active (toggle))
+    return;
+
+  accent_color_from_key = g_settings_get_enum (self->interface_settings,
+                                               INTERFACE_ACCENT_COLOR_KEY);
+
+  /* Don't unnecessarily set the key again */
+  if (accent_color == accent_color_from_key)
+    return;
+
+  transition_screen (self);
+
+  g_settings_set_enum (self->interface_settings,
+                       INTERFACE_ACCENT_COLOR_KEY,
+                       accent_color);
+}
+
+/* Adapted from adw-inspector-page.c */
+static const char *
+get_color_tooltip (GDesktopAccentColor color)
+{
+  switch (color)
+    {
+    case G_DESKTOP_ACCENT_COLOR_BLUE:
+      return _("Blue");
+    case G_DESKTOP_ACCENT_COLOR_TEAL:
+      return _("Teal");
+    case G_DESKTOP_ACCENT_COLOR_GREEN:
+      return _("Green");
+    case G_DESKTOP_ACCENT_COLOR_YELLOW:
+      return _("Yellow");
+    case G_DESKTOP_ACCENT_COLOR_ORANGE:
+      return _("Orange");
+    case G_DESKTOP_ACCENT_COLOR_RED:
+      return _("Red");
+    case G_DESKTOP_ACCENT_COLOR_PINK:
+      return _("Pink");
+    case G_DESKTOP_ACCENT_COLOR_PURPLE:
+      return _("Purple");
+    case G_DESKTOP_ACCENT_COLOR_SLATE:
+      return _("Slate");
+    default:
+      g_assert_not_reached ();
+    }
+}
+
+static const char *
+get_untranslated_color (GDesktopAccentColor color)
+{
+  switch (color)
+    {
+    case G_DESKTOP_ACCENT_COLOR_BLUE:
+      return "blue";
+    case G_DESKTOP_ACCENT_COLOR_TEAL:
+      return "teal";
+    case G_DESKTOP_ACCENT_COLOR_GREEN:
+      return "green";
+    case G_DESKTOP_ACCENT_COLOR_YELLOW:
+      return "yellow";
+    case G_DESKTOP_ACCENT_COLOR_ORANGE:
+      return "orange";
+    case G_DESKTOP_ACCENT_COLOR_RED:
+      return "red";
+    case G_DESKTOP_ACCENT_COLOR_PINK:
+      return "pink";
+    case G_DESKTOP_ACCENT_COLOR_PURPLE:
+      return "purple";
+    case G_DESKTOP_ACCENT_COLOR_SLATE:
+      return "slate";
+    default:
+      g_assert_not_reached ();
+    }
+}
+
+static void
+setup_accent_color_toggles (CcBackgroundPanel *self)
+{
+  GDesktopAccentColor accent_color = g_settings_get_enum (self->interface_settings, INTERFACE_ACCENT_COLOR_KEY);
+  GDesktopAccentColor i;
+
+  for (i = G_DESKTOP_ACCENT_COLOR_BLUE; i <= G_DESKTOP_ACCENT_COLOR_SLATE; i++)
+    {
+      GtkWidget *button = GTK_WIDGET (gtk_toggle_button_new ());
+      GtkToggleButton *grouping_button = GTK_TOGGLE_BUTTON (gtk_widget_get_first_child (self->accent_box));
+
+      gtk_widget_set_tooltip_text (button, get_color_tooltip (i));
+      gtk_widget_add_css_class (button, "accent-button");
+      gtk_widget_add_css_class (button, get_untranslated_color (i));
+      g_object_set_data (G_OBJECT (button), "accent-color", GINT_TO_POINTER (i));
+      g_signal_connect_object (button, "toggled",
+                               G_CALLBACK (on_accent_color_toggled_cb),
+                               self,
+                               G_CONNECT_SWAPPED);
+
+      if (grouping_button != NULL)
+        gtk_toggle_button_set_group (GTK_TOGGLE_BUTTON (button), grouping_button);
+
+      if (i == accent_color)
+        gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button), TRUE);
+
+      gtk_box_append (GTK_BOX (self->accent_box), button);
+    }
+}
+
+static void
+reload_accent_color_toggles (CcBackgroundPanel *self)
+{
+  GDesktopAccentColor accent_color = g_settings_get_enum (self->interface_settings, INTERFACE_ACCENT_COLOR_KEY);
+  GtkWidget *child;
+
+  for (child = gtk_widget_get_first_child (self->accent_box);
+       child;
+       child = gtk_widget_get_next_sibling (child))
+    {
+      GDesktopAccentColor child_color = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (child), "accent-color"));
+
+      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (child), child_color == accent_color);
+    }
+}
+
+static void
+reload_color_scheme (CcBackgroundPanel *self)
+{
+  GDesktopColorScheme scheme;
+
+  scheme = g_settings_get_enum (self->interface_settings, INTERFACE_COLOR_SCHEME_KEY);
+
+  if (scheme == G_DESKTOP_COLOR_SCHEME_DEFAULT)
+    {
+      gtk_toggle_button_set_active (self->default_toggle, TRUE);
+    }
+  else if (scheme == G_DESKTOP_COLOR_SCHEME_PREFER_DARK)
+    {
+      gtk_toggle_button_set_active (self->dark_toggle, TRUE);
+    }
+  else
+    {
+      gtk_toggle_button_set_active (self->default_toggle, FALSE);
+      gtk_toggle_button_set_active (self->dark_toggle, FALSE);
+    }
+}
+
 static void
 set_color_scheme (CcBackgroundPanel   *self,
                   GDesktopColorScheme  color_scheme)
@@ -185,20 +315,9 @@ update_preview (CcBackgroundPanel *self)
   cc_background_preview_set_item (self->dark_preview, current_background);
 }
 
-static gchar *
-get_save_path (void)
-{
-  return g_build_filename (g_get_user_config_dir (),
-                           "regolith-control-center",
-                           "backgrounds",
-                           "last-edited.xml",
-                           NULL);
-}
-
 static void
 reload_current_bg (CcBackgroundPanel *self)
 {
-  g_autoptr(CcBackgroundItem) saved = NULL;
   CcBackgroundItem *configured;
   GSettings *settings = NULL;
   g_autofree gchar *uri = NULL;
@@ -206,10 +325,6 @@ reload_current_bg (CcBackgroundPanel *self)
   g_autofree gchar *pcolor = NULL;
   g_autofree gchar *scolor = NULL;
 
-  /* Load the saved configuration */
-  uri = get_save_path ();
-  saved = cc_background_xml_get_item (uri);
-
   /* initalise the current background information from settings */
   settings = self->settings;
   uri = g_settings_get_string (settings, WP_URI_KEY);
@@ -231,41 +346,11 @@ reload_current_bg (CcBackgroundPanel *self)
                 "secondary-color", scolor,
                 NULL);
 
-  if (saved != NULL && cc_background_item_compare (saved, configured))
-    {
-      CcBackgroundItemFlags flags;
-      flags = cc_background_item_get_flags (saved);
-      /* Special case for colours */
-      if (cc_background_item_get_placement (saved) == G_DESKTOP_BACKGROUND_STYLE_NONE)
-        flags &=~ (CC_BACKGROUND_ITEM_HAS_PCOLOR | CC_BACKGROUND_ITEM_HAS_SCOLOR);
-      g_object_set (G_OBJECT (configured),
-		    "name", cc_background_item_get_name (saved),
-		    "flags", flags,
-		    "source-url", cc_background_item_get_source_url (saved),
-		    "source-xml", cc_background_item_get_source_xml (saved),
-		    NULL);
-    }
-
   g_clear_object (&self->current_background);
   self->current_background = configured;
   cc_background_item_load (configured, NULL);
-}
 
-static gboolean
-create_save_dir (void)
-{
-  g_autofree char *path = NULL;
-
-  path = g_build_filename (g_get_user_config_dir (),
-			   "regolith-control-center",
-			   "backgrounds",
-			   NULL);
-  if (g_mkdir_with_parents (path, USER_DIR_MODE) < 0)
-    {
-      g_warning ("Failed to create directory '%s'", path);
-      return FALSE;
-    }
-  return TRUE;
+  cc_background_chooser_set_active_item (self->background_chooser, configured);
 }
 
 static void
@@ -370,11 +455,6 @@ set_background (CcBackgroundPanel *self,
 
   /* Clean out dconf if the user went back to distro defaults */
   reset_settings_if_defaults (self, settings, set_dark);
-
-  /* Save the source XML if there is one */
-  filename = get_save_path ();
-  if (create_save_dir ())
-    cc_background_xml_save (self->current_background, filename);
 }
 
 static void
@@ -409,7 +489,13 @@ static void
 on_chooser_background_chosen_cb (CcBackgroundPanel *self,
                                  CcBackgroundItem  *item)
 {
+  g_signal_handlers_block_by_func (self->settings, on_settings_changed, self);
+
   set_background_regolith (item);
+  
+  on_settings_changed (self);
+
+  g_signal_handlers_unblock_by_func (self->settings, on_settings_changed, self);
 }
 
 static void
@@ -464,6 +550,7 @@ cc_background_panel_class_init (CcBackgroundPanelClass *klass)
 
   gtk_widget_class_set_template_from_resource (widget_class, "/org/gnome/control-center/background/cc-background-panel.ui");
 
+  gtk_widget_class_bind_template_child (widget_class, CcBackgroundPanel, accent_box);
   gtk_widget_class_bind_template_child (widget_class, CcBackgroundPanel, background_chooser);
   gtk_widget_class_bind_template_child (widget_class, CcBackgroundPanel, default_preview);
   gtk_widget_class_bind_template_child (widget_class, CcBackgroundPanel, dark_preview);
@@ -507,11 +594,18 @@ cc_background_panel_init (CcBackgroundPanel *self)
   g_signal_connect_object (self->settings, "changed", G_CALLBACK (on_settings_changed), self, G_CONNECT_SWAPPED);
 
   /* Interface settings */
-  reload_color_scheme_toggles (self);
+  reload_color_scheme (self);
+  setup_accent_color_toggles (self);
 
   g_signal_connect_object (self->interface_settings,
                            "changed::" INTERFACE_COLOR_SCHEME_KEY,
-                           G_CALLBACK (reload_color_scheme_toggles),
+                           G_CALLBACK (reload_color_scheme),
+                           self,
+                           G_CONNECT_SWAPPED);
+
+  g_signal_connect_object (self->interface_settings,
+                           "changed::" INTERFACE_ACCENT_COLOR_KEY,
+                           G_CALLBACK (reload_accent_color_toggles),
                            self,
                            G_CONNECT_SWAPPED);
 
