@@ -30,7 +30,7 @@
 #include "cc-keyboard-resources.h"
 #include "cc-keyboard-shortcut-dialog.h"
 #include "cc-input-list-box.h"
-#include "cc-xkb-modifier-dialog.h"
+#include "cc-xkb-modifier-page.h"
 #include "cc-list-row.h"
 
 #include "keyboard-shortcuts.h"
@@ -63,6 +63,10 @@ static const CcXkbModifier LV3_MODIFIER = {
   N_("Alternate Characters Key"),
   N_("The alternate characters key can be used to enter additional characters. These are sometimes printed as a third-option on your keyboard."),
   (CcXkbOption[]){
+    /* Set xkb_option to the prefix, namely "lvl3:", to signify removal of any
+     * lvl3: option from the xkb-options Gsetting.
+     */
+    { NC_("keyboard key", "Default"), "lv3:" },
     { NC_("keyboard key", "None"),        "lv3:ralt_alt" },
     { NC_("keyboard key", "Left Alt"),    "lv3:lalt_switch" },
     { NC_("keyboard key", "Right Alt"),   "lv3:ralt_switch" },
@@ -72,68 +76,60 @@ static const CcXkbModifier LV3_MODIFIER = {
     { NC_("keyboard key", "Right Ctrl"),  "lv3:switch" },
     { NULL,                               NULL }
   },
-  "lv3:ralt_switch",
-  "lv3:ralt_alt",
-  NULL,
+  "lv3:",
 };
 
 static const CcXkbModifier COMPOSE_MODIFIER = {
   "compose:",
   N_("Compose Key"),
   N_("The compose key allows a wide variety of characters to be entered. To use it, press compose then a sequence of characters. "
-     " For example, compose key followed by <b>C</b> and <b>o</b> will enter <b>©</b>, "
+     " For example, compose key followed by <b>o</b> and <b>c</b> will enter <b>©</b>, "
      "<b>a</b> followed by <b>'</b> will enter <b>á</b>."),
   (CcXkbOption[]){
     { NC_("keyboard key", "Right Alt"),    "compose:ralt" },
     { NC_("keyboard key", "Left Super"),   "compose:lwin" },
     { NC_("keyboard key", "Right Super"),  "compose:rwin" },
     { NC_("keyboard key", "Menu key"),     "compose:menu" },
+    { NC_("keyboard key", "Left Ctrl"),    "compose:lctrl" },
     { NC_("keyboard key", "Right Ctrl"),   "compose:rctrl" },
     { NC_("keyboard key", "Caps Lock"),    "compose:caps" },
     { NC_("keyboard key", "Scroll Lock"),  "compose:sclk" },
     { NC_("keyboard key", "Print Screen"), "compose:prsc" },
+    { NC_("keyboard key", "Insert"),       "compose:ins" },
     { NULL,                                NULL }
   },
   NULL,
-  NULL,
 };
 
 static void
-show_modifier_dialog (CcKeyboardPanel *self, const CcXkbModifier *modifier)
+show_modifier_page (CcKeyboardPanel *self, const CcXkbModifier *modifier)
 {
-  GtkWindow *window, *dialog;
+  AdwNavigationPage *page;
 
-  window = GTK_WINDOW (cc_shell_get_toplevel (cc_panel_get_shell (CC_PANEL (self))));
+  page = ADW_NAVIGATION_PAGE (cc_xkb_modifier_page_new (self->input_source_settings, modifier));
 
-  dialog = GTK_WINDOW (cc_xkb_modifier_dialog_new (self->input_source_settings, modifier));
-
-  gtk_window_set_transient_for (dialog, window);
-  gtk_window_present (dialog);
+  cc_panel_push_subpage (CC_PANEL (self), page);
 }
 
 static void
 alt_chars_row_activated (CcKeyboardPanel *self)
 {
-  show_modifier_dialog (self, &LV3_MODIFIER);
+  show_modifier_page (self, &LV3_MODIFIER);
 }
 
 static void
 compose_row_activated (CcKeyboardPanel *self)
 {
-  show_modifier_dialog (self, &COMPOSE_MODIFIER);
+  show_modifier_page (self, &COMPOSE_MODIFIER);
 }
 
 static void
 keyboard_shortcuts_activated (CcKeyboardPanel *self)
 {
-  GtkWindow *window;
-  GtkWidget *shortcut_dialog;
+  AdwDialog *dialog;
 
-  window = GTK_WINDOW (cc_shell_get_toplevel (cc_panel_get_shell (CC_PANEL (self))));
-
-  shortcut_dialog = cc_keyboard_shortcut_dialog_new ();
-  gtk_window_set_transient_for (GTK_WINDOW (shortcut_dialog), window);
-  gtk_window_present (GTK_WINDOW (shortcut_dialog));
+  dialog = ADW_DIALOG (cc_keyboard_shortcut_dialog_new ());
+  adw_dialog_present (dialog, GTK_WIDGET (self));
 }
 
 static void
