@@ -49,6 +49,10 @@ static void launch_panel_activated (GSimpleAction *action,
                                     GVariant      *parameter,
                                     gpointer       user_data);
 
+static void launch_single_panel_mode_activated (GSimpleAction *action,
+                                                GVariant      *parameter,
+                                                gpointer       user_data);
+
 static void help_activated         (GSimpleAction *action,
                                     GVariant      *parameter,
                                     gpointer       user_data);
@@ -66,7 +70,7 @@ G_DEFINE_TYPE (CcApplication, cc_application, ADW_TYPE_APPLICATION)
 
 const GOptionEntry all_options[] = {
   { "version", 0, 0, G_OPTION_ARG_NONE, NULL, N_("Display version number"), NULL },
-  { "verbose", 'v', G_OPTION_FLAG_NO_ARG, G_OPTION_ARG_CALLBACK, cmd_verbose_cb, N_("Enable verbose mode"), NULL },
+  { "verbose", 'v', G_OPTION_FLAG_NO_ARG, G_OPTION_ARG_CALLBACK, cmd_verbose_cb, N_("Enable verbose mode. Specify multiple times to increase verbosity"), NULL },
   { "search", 's', 0, G_OPTION_ARG_STRING, NULL, N_("Search for the string"), "SEARCH" },
   { "list", 'l', 0, G_OPTION_ARG_NONE, NULL, N_("List possible panel names and exit"), NULL },
   { G_OPTION_REMAINING, '\0', 0, G_OPTION_ARG_FILENAME_ARRAY, NULL, N_("Panel to display"), N_("[PANEL] [ARGUMENT…]") },
@@ -75,6 +79,7 @@ const GOptionEntry all_options[] = {
 
 static const GActionEntry cc_app_actions[] = {
   { "launch-panel", launch_panel_activated, "(sav)", NULL, NULL, { 0 } },
+  { "launch-single-panel-mode", launch_single_panel_mode_activated, "(sav)", NULL, NULL, { 0 } },
   { "help", help_activated, NULL, NULL, NULL, { 0 } },
   { "quit", cc_application_quit, NULL, NULL, NULL, { 0 } },
   { "about", about_activated, NULL, NULL, NULL, { 0 } }
@@ -88,6 +93,7 @@ help_activated (GSimpleAction *action,
   CcApplication *self = CC_APPLICATION (user_data);
   CcPanel *panel;
   GtkWidget *window;
+  g_autoptr(GtkUriLauncher) launcher = NULL;
   const char *uri = NULL;
 
   panel = cc_shell_get_active_panel (CC_SHELL (self->window));
@@ -95,9 +101,9 @@ help_activated (GSimpleAction *action,
     uri = cc_panel_get_help_uri (panel);
 
   window = cc_shell_get_toplevel (CC_SHELL (self->window));
-  gtk_show_uri (GTK_WINDOW (window),
-                uri ? uri : "help:gnome-help/prefs",
-                GDK_CURRENT_TIME);
+
+  launcher = gtk_uri_launcher_new (uri ? uri : "help:gnome-help/prefs");
+  gtk_uri_launcher_launch (launcher, GTK_WINDOW (window), NULL, NULL, NULL);
 }
 
 static void
@@ -107,18 +113,17 @@ about_activated (GSimpleAction *action,
 
 {
   CcApplication *self = CC_APPLICATION (user_data);
-  GtkWidget *about_window;
+  AdwDialog *about_dialog;
   const char *developer_name;
 
-  about_window = adw_about_window_new_from_appdata ("/org/gnome/Settings/appdata", NULL);
-  adw_about_window_set_version (ADW_ABOUT_WINDOW (about_window), VERSION);
-  developer_name = adw_about_window_get_developer_name (ADW_ABOUT_WINDOW (about_window));
+  about_dialog = adw_about_dialog_new_from_appdata ("/org/gnome/Settings/appdata", NULL);
+  adw_about_dialog_set_version (ADW_ABOUT_DIALOG (about_dialog), VERSION);
+  developer_name = adw_about_dialog_get_developer_name (ADW_ABOUT_DIALOG (about_dialog));
   /* Translators should localize the following string which will be displayed in the About dialog giving credit to the translator(s). */
-  adw_about_window_set_translator_credits (ADW_ABOUT_WINDOW (about_window), _("translator-credits"));
-  adw_about_window_set_copyright (ADW_ABOUT_WINDOW (about_window), g_strdup_printf (_("© 1998 %s"), developer_name));
-  gtk_window_set_transient_for (GTK_WINDOW (about_window), GTK_WINDOW (self->window));
+  adw_about_dialog_set_translator_credits (ADW_ABOUT_DIALOG (about_dialog), _("translator-credits"));
+  adw_about_dialog_set_copyright (ADW_ABOUT_DIALOG (about_dialog), g_strdup_printf (_("© 1998 %s"), developer_name));
 
-  gtk_window_present (GTK_WINDOW (about_window));
+  adw_dialog_present (about_dialog, GTK_WIDGET (self->window));
 }
 
 static gboolean
@@ -155,6 +160,18 @@ launch_panel_activated (GSimpleAction *action,
   g_application_activate (G_APPLICATION (self));
 }
 
+static void
+launch_single_panel_mode_activated (GSimpleAction *action,
+                                    GVariant      *parameter,
+                                    gpointer       user_data)
+{
+  CcApplication *self = CC_APPLICATION (user_data);
+
+  cc_window_enable_single_panel_mode (self->window);
+
+  launch_panel_activated (action, parameter, user_data);
+}
+
 static char **
 get_current_desktops (void)
 {
@@ -303,6 +320,8 @@ cc_application_startup (GApplication *application)
 
   G_APPLICATION_CLASS (cc_application_parent_class)->startup (application);
 
+  gtk_window_set_default_icon_name (APPLICATION_ID);
+
   gtk_application_set_accels_for_action (GTK_APPLICATION (application),
                                          "app.help", help_accels);
 
@@ -370,7 +389,7 @@ GtkApplication *
 cc_application_new (void)
 {
   return g_object_new (CC_TYPE_APPLICATION,
-                       "application-id", "org.regolith.Settings",
+                       "application-id", APPLICATION_ID,
                        "flags", G_APPLICATION_HANDLES_COMMAND_LINE,
                        NULL);
 }
