@@ -24,6 +24,7 @@
 
 #include "cc-log.h"
 #include "cc-panel-list.h"
+#include "cc-ui-util.h"
 #include "cc-util.h"
 
 typedef struct
@@ -53,6 +54,7 @@ struct _CcPanelList
 
   gchar              *current_panel_id;
   gchar              *search_query;
+  gchar             **search_words;
 
   CcPanelListView     previous_view;
   CcPanelListView     view;
@@ -261,7 +263,10 @@ row_data_new (CcPanelCategory     category,
   gtk_grid_set_column_spacing (GTK_GRID (grid), 12);
 
   /* Icon */
-  image = gtk_image_new_from_icon_name (icon);
+  image = g_object_new (GTK_TYPE_IMAGE,
+                        "accessible-role", GTK_ACCESSIBLE_ROLE_PRESENTATION,
+                        "icon-name", icon,
+                        NULL);
 
   gtk_grid_attach (GTK_GRID (grid), image, 0, 0, 1, 1);
 
@@ -312,24 +317,21 @@ filter_func (GtkListBoxRow *row,
 {
   CcPanelList *self;
   RowData *data;
-  g_autofree gchar *search_text = NULL;
   g_autofree gchar *panel_text = NULL;
   g_autofree gchar *panel_description = NULL;
-  gboolean retval = FALSE;
-  gint i;
+  gboolean retval = TRUE;
+  gint i, j;
 
   self = CC_PANEL_LIST (user_data);
   data = g_object_get_data (G_OBJECT (row), "data");
 
-  if (!self->search_query)
+  if (!self->search_words)
     return TRUE;
 
   panel_text = cc_util_normalize_casefold_and_unaccent (data->name);
-  search_text = cc_util_normalize_casefold_and_unaccent (self->search_query);
   panel_description = cc_util_normalize_casefold_and_unaccent (data->description);
 
   g_strstrip (panel_text);
-  g_strstrip (search_text);
   g_strstrip (panel_description);
 
   /*
@@ -338,11 +340,24 @@ filter_func (GtkListBoxRow *row,
    */
   gtk_widget_set_visible (data->description_label, self->view == CC_PANEL_LIST_SEARCH);
 
-  for (i = 0; !retval && data->keywords[i] != NULL; i++)
-      retval = (strstr (data->keywords[i], search_text) == data->keywords[i]);
+  for (j = 0; retval && self->search_words[j] != NULL; j++) {
+    const gchar *search_word = self->search_words[j];
+    gboolean match = FALSE;
 
-  retval = retval || g_strstr_len (panel_text, -1, search_text) != NULL ||
-           g_strstr_len (panel_description, -1, search_text) != NULL;
+    if (search_word[0] == '\0')
+      continue;
+
+    // Compare keywords
+    for (i = 0; !match && data->keywords[i] != NULL; i++)
+      match = (strstr (data->keywords[i], search_word) == data->keywords[i]);
+
+    // Compare panel title and description
+    match = match || (g_strstr_len (panel_text, -1, search_word) != NULL ||
+                      g_strstr_len (panel_description, -1, search_word) != NULL);
+
+    // All search words must match
+    retval = retval && match;
+  }
 
   return retval;
 }
@@ -360,13 +375,11 @@ static const gchar * const panel_order[] = {
   "display",
   "sound",
   "power",
-  "multitasking",
   "background",
 
   "separator",
 
   "applications",
-  "notifications",
   "search",
   "online-accounts",
   "sharing",
@@ -415,6 +428,9 @@ sort_function (GtkListBoxRow *a,
   return get_panel_id_index (a_id) - get_panel_id_index (b_id);
 }
 
+
+/* FIXME: This is now different from the "match all words" search.
+          Maybe add a search score based on number of matches in filter_func()? */
 static gint
 search_sort_function (GtkListBoxRow *a,
                       GtkListBoxRow *b,
@@ -473,7 +489,7 @@ header_func (GtkListBoxRow *row,
     return;
 
   pid = get_panel_id_index (get_panel_id_from_row (user_data, row));
-  if (g_str_equal (panel_order[pid-1], "separator"))
+  if (pid > 0 && g_str_equal (panel_order[pid-1], "separator"))
     {
       GtkWidget *separator;
 
@@ -497,6 +513,7 @@ row_activated_cb (GtkWidget     *listbox,
                   CcPanelList   *self)
 {
   RowData *data;
+  const gchar *parent_panel = 0;
 
   /*
    * Since we're not sure that the activated row is in the
@@ -505,8 +522,12 @@ row_activated_cb (GtkWidget     *listbox,
   switch_to_view (self, get_view_from_listbox (self, listbox));
 
   data = g_object_get_data (G_OBJECT (row), "data");
+  if (data->category == CC_CATEGORY_SYSTEM)
+    parent_panel = "system";
+  else if (data->category == CC_CATEGORY_PRIVACY)
+    parent_panel = "privacy";
 
-  g_signal_emit (self, signals[SHOW_PANEL], 0, data->id);
+  g_signal_emit (self, signals[SHOW_PANEL], 0, data->id, parent_panel);
 
   /* After selecting the panel and eventually changing the view, reset the
    * autoselect flag. If necessary, cc_panel_list_set_active_panel() will
@@ -573,6 +594,7 @@ cc_panel_list_finalize (GObject *object)
   CcPanelList *self = (CcPanelList *)object;
 
   g_clear_pointer (&self->search_query, g_free);
+  g_clear_pointer (&self->search_words, g_strfreev);
   g_clear_pointer (&self->current_panel_id, g_free);
   g_clear_pointer (&self->id_to_data, g_hash_table_destroy);
   g_clear_pointer (&self->id_to_search_data, g_hash_table_destroy);
@@ -634,28 +656,6 @@ cc_panel_list_set_property (GObject      *object,
     }
 }
 
-static gboolean
-search_list_keynav_failed_cb (CcPanelList *self,
-                              GtkDirectionType direction)
-{
-  GtkWidget *toplevel;
-
-  /* We are in the first result of search list and pressing Arrow Up,
-   * so then we move focus back to search text entry */
-  if (direction == GTK_DIR_UP)
-    {
-      toplevel = GTK_WIDGET (gtk_widget_get_root (GTK_WIDGET (self)));
-
-      if (!toplevel)
-        return FALSE;
-
-      return gtk_widget_child_focus (toplevel, GTK_DIR_TAB_BACKWARD);
-    }
-
-  return FALSE;
-}
-
-
 static void
 cc_panel_list_class_init (CcPanelListClass *klass)
 {
@@ -676,7 +676,8 @@ cc_panel_list_class_init (CcPanelListClass *klass)
                                       G_SIGNAL_RUN_LAST,
                                       0, NULL, NULL, NULL,
                                       G_TYPE_NONE,
-                                      1,
+                                      2,
+                                      G_TYPE_STRING,
                                       G_TYPE_STRING);
 
   /**
@@ -722,9 +723,9 @@ cc_panel_list_class_init (CcPanelListClass *klass)
   gtk_widget_class_bind_template_child (widget_class, CcPanelList, search_listbox);
   gtk_widget_class_bind_template_child (widget_class, CcPanelList, stack);
 
+  gtk_widget_class_bind_template_callback (widget_class, cc_util_keynav_propagate_up);
   gtk_widget_class_bind_template_callback (widget_class, row_activated_cb);
   gtk_widget_class_bind_template_callback (widget_class, search_row_activated_cb);
-  gtk_widget_class_bind_template_callback (widget_class, search_list_keynav_failed_cb);
 }
 
 static void
@@ -823,9 +824,17 @@ cc_panel_list_set_search_query (CcPanelList *self,
 
   if (g_strcmp0 (self->search_query, search) != 0)
     {
+      g_autofree gchar *search_query_normalized;
+
       g_clear_pointer (&self->search_query, g_free);
+      g_clear_pointer (&self->search_words, g_strfreev);
+
       self->search_query = g_strdup (search);
 
+      /* Split on spaces */
+      search_query_normalized = cc_util_normalize_casefold_and_unaccent (search);
+      self->search_words = g_strsplit (g_strstrip (search_query_normalized), " ", 0);
+
       update_search (self);
 
       g_object_notify_by_pspec (G_OBJECT (self), properties[PROP_SEARCH_QUERY]);
