@@ -41,12 +41,10 @@ extern GType cc_date_time_panel_get_type (void);
 extern GType cc_display_panel_get_type (void);
 extern GType cc_keyboard_panel_get_type (void);
 extern GType cc_mouse_panel_get_type (void);
-extern GType cc_multitasking_panel_get_type (void);
 #ifdef BUILD_NETWORK
 extern GType cc_network_panel_get_type (void);
 extern GType cc_wifi_panel_get_type (void);
 #endif /* BUILD_NETWORK */
-extern GType cc_notifications_panel_get_type (void);
 extern GType cc_online_accounts_panel_get_type (void);
 extern GType cc_power_panel_get_type (void);
 extern GType cc_printers_panel_get_type (void);
@@ -67,6 +65,7 @@ extern GType cc_wwan_panel_get_type (void);
 #ifdef BUILD_NETWORK
 extern void cc_wifi_panel_static_init_func (void);
 #endif /* BUILD_NETWORK */
+extern void cc_sharing_panel_static_init_func (void);
 #ifdef BUILD_WACOM
 extern void cc_wacom_panel_static_init_func (void);
 #endif /* BUILD_WACOM */
@@ -84,7 +83,7 @@ extern void cc_wwan_panel_static_init_func (void);
 
 static CcPanelLoaderVtable default_panels[] =
 {
-//  PANEL_TYPE("applications",     cc_applications_panel_get_type,         NULL),
+  PANEL_TYPE("applications",     cc_applications_panel_get_type,         NULL),
   PANEL_TYPE("background",       cc_background_panel_get_type,           NULL),
 #ifdef BUILD_BLUETOOTH
   PANEL_TYPE("bluetooth",        cc_bluetooth_panel_get_type,            NULL),
@@ -93,18 +92,16 @@ static CcPanelLoaderVtable default_panels[] =
   PANEL_TYPE("display",          cc_display_panel_get_type,              NULL),
   PANEL_TYPE("keyboard",         cc_keyboard_panel_get_type,             NULL),
   PANEL_TYPE("mouse",            cc_mouse_panel_get_type,                NULL),
-//  PANEL_TYPE("multitasking",     cc_multitasking_panel_get_type,         NULL),
 #ifdef BUILD_NETWORK
   PANEL_TYPE("network",          cc_network_panel_get_type,              NULL),
   PANEL_TYPE("wifi",             cc_wifi_panel_get_type,                 cc_wifi_panel_static_init_func),
 #endif
-//  PANEL_TYPE("notifications",    cc_notifications_panel_get_type,        NULL),
   PANEL_TYPE("online-accounts",  cc_online_accounts_panel_get_type,      NULL),
   PANEL_TYPE("power",            cc_power_panel_get_type,                NULL),
   PANEL_TYPE("printers",         cc_printers_panel_get_type,             NULL),
   PANEL_TYPE("privacy",          cc_privacy_panel_get_type,              NULL),
-//  PANEL_TYPE("search",           cc_search_panel_get_type,               NULL),
-  PANEL_TYPE("sharing",          cc_sharing_panel_get_type,              NULL),
+  PANEL_TYPE("search",           cc_search_panel_get_type,               NULL),
+  PANEL_TYPE("sharing",          cc_sharing_panel_get_type,              cc_sharing_panel_static_init_func),
   PANEL_TYPE("sound",            cc_sound_panel_get_type,                NULL),
   PANEL_TYPE("system",           cc_system_panel_get_type,               NULL),
   PANEL_TYPE("universal-access", cc_ua_panel_get_type,                   NULL),
@@ -112,7 +109,7 @@ static CcPanelLoaderVtable default_panels[] =
   PANEL_TYPE("wacom",            cc_wacom_panel_get_type,                cc_wacom_panel_static_init_func),
 #endif
 #ifdef BUILD_WWAN
-//  PANEL_TYPE("wwan",             cc_wwan_panel_get_type,                 cc_wwan_panel_static_init_func),
+  PANEL_TYPE("wwan",             cc_wwan_panel_get_type,                 cc_wwan_panel_static_init_func),
 #endif
 };
 
@@ -122,6 +119,21 @@ static CcPanelLoaderVtable default_panels[] =
 static CcPanelLoaderVtable *panels_vtable = default_panels;
 static gsize panels_vtable_len = G_N_ELEMENTS (default_panels);
 
+typedef struct
+{
+  CcPanelCategory category;
+  const gchar *name;
+} CcSubpageLoaderVtable;
+
+static CcSubpageLoaderVtable default_subpages[] =
+{
+  {CC_CATEGORY_SYSTEM, "about"},
+  {CC_CATEGORY_SYSTEM, "datetime"},
+  {CC_CATEGORY_SYSTEM, "region"},
+  {CC_CATEGORY_SYSTEM, "users"},
+};
+static CcSubpageLoaderVtable *subpages_vtable = default_subpages;
+static gsize supages_vtable_len = G_N_ELEMENTS (default_subpages);
 
 static int
 parse_categories (GDesktopAppInfo *app)
@@ -147,6 +159,10 @@ parse_categories (GDesktopAppInfo *app)
     retval = CC_CATEGORY_DEVICES;
   else if (g_strv_contains (const_strv (split), "X-GNOME-DetailsSettings"))
     retval = CC_CATEGORY_DETAILS;
+  else if (g_strv_contains (const_strv (split), "X-GNOME-SystemSettings"))
+    retval = CC_CATEGORY_SYSTEM;
+  else if (g_strv_contains (const_strv (split), "X-GNOME-PrivacySettings"))
+    retval = CC_CATEGORY_PRIVACY;
   else if (g_strv_contains (const_strv (split), "HardwareSettings"))
     retval = CC_CATEGORY_HARDWARE;
 
@@ -245,6 +261,24 @@ cc_panel_loader_fill_model (CcShellModel *model)
       cc_shell_model_add_item (model, category, G_APP_INFO (app), panels_vtable[i].name);
     }
 
+  for (i = 0; i < supages_vtable_len; i++)
+    {
+      g_autoptr(GDesktopAppInfo) app = NULL;
+      g_autofree gchar *desktop_name = NULL;
+
+      desktop_name = g_strconcat ("gnome-", subpages_vtable[i].name, "-panel.desktop", NULL);
+      app = g_desktop_app_info_new (desktop_name);
+
+      if (!app)
+        {
+          g_warning ("Ignoring broken panel %s (missing desktop file)", subpages_vtable[i].name);
+          continue;
+        }
+
+      cc_shell_model_add_item (model, subpages_vtable[i].category, G_APP_INFO (app), subpages_vtable[i].name);
+      cc_shell_model_set_panel_visibility (model, subpages_vtable[i].name, CC_PANEL_VISIBLE_IN_SEARCH);
+    }
+
   /* If there's an static init function, execute it after adding all panels to
    * the model. This will allow the panels to show or hide themselves without
    * having an instance running.
